# coding: utf-8 -*-


# imports
from . basic import *
from . case_studies import *
from . samples import *
from . test_suite import *
from . pir_figures import *
from . figures import *
from . risk_progression import *
from . bodoff import *
from . ft import *

# module level doc-string
__doc__ = """
Extensions contains optional, nice to have code that extends basic functionality.
For example, all CaseStudy material is included here. It is not required for core
functionality. It is not included in the default import. It is included in the
default build.
"""
